/** @ragempcommunity */
export declare type PlayerMp = any

export declare type BrowserEventNames = 'updatePlayerData' | 'triggerSomething'
export declare type ClientEventNames = 'updatePlayerData' | 'triggerSomething'
export declare type ServerEventNames = 'playerJoin' | 'updatePlayerData' | 'triggerSomething'

// ======================================

/**
 * NOT INTENDED FOR OUT-OF-CONTEXT USE
 */
export declare class Browser extends Wrapper {
    constructor(options?: RpcWrapperConfig);
    /**
     * NOT INTENDED FOR OUT-OF-CONTEXT USE
     */
    _resolveEmitDestination(dataRaw: string): void;
    private emitClient;
    private emit;
}

/**
 * NOT INTENDED FOR OUT-OF-CONTEXT USE
 */
export declare class Client extends Wrapper {
    private _browser;
    constructor(options?: RpcWrapperConfig);
    set browser(browser: any);
    /**
     * NOT INTENDED FOR OUT-OF-CONTEXT USE
     */
    _resolveEmitDestination(dataRaw: string): void;
    private emit;
    private emitServer;
    private emitBrowser;
    private errorNoBrowser;
}

export declare enum Environment {
    BROWSER = "BROWSER",
    CLIENT = "CLIENT",
    SERVER = "SERVER",
    UNKNOWN = "UNKNOWN"
}

export declare enum Errors {
    EVENT_NOT_REGISTERED = "Event not registered",
    UNKNOWN_ENVIRONMENT = "Unknown environment",
    NO_BROWSER = "You need to initialize browser first",
    EVENT_RESPONSE_TIMEOUT = "Response was timed out after 10s of inactivity"
}

export declare enum Events {
    LOCAL_EVENT_LISTENER = "__rpc:listener",
    SERVER_EVENT_LISTENER = "__rpc:serverListener",
    EVENT_RESPONSE = "__rpc:response"
}

export declare const nativeClientEvents: Set<string>;

export declare const nativeServerEvents: Set<string>;

export declare class Rpc extends Wrapper {
private _server;
private _client;
private _browser;
constructor(options?: RpcConfig);
set browser(browser: any);
/**
* Registers a callback function for a specified event
*
* @template CallbackArguments - An array of argument types that the callback function accepts
* @template CallbackReturn - The type of the value returned by the callback function
* @template EventName - A string or union representing the event name
*
* @param {EventName} eventName - The name of the event to register the callback for
* @param {(...args: CallbackArguments) => CallbackReturn} cb - The callback function that is called when the event is triggered
*
* @returns {void}
*
* @example
* register<[PlayerMp]>('playerJoin', (player) => {
*   console.log(`Connected: ${player.socialClub}`)
* })
*/
register<CallbackArguments extends unknown[] = unknown[], CallbackReturn = unknown, EventName extends string = string>(eventName: EventName, cb: (...args: CallbackArguments) => CallbackReturn): void;
/**
* Unregisters callback function for a specified event
*
* @template EventName - A string or union representing the event name
*
* @param {EventName} eventName - The name of the event to register the callback for
*
* @returns {void}
*
* @example
* unregister('playerJoin')
*/
unregister<EventName extends string = string>(eventName: EventName): void;
/**
* Calls a client-side event from browser or server. Use 'call' to call client from client
*
* @template Arguments - An array of argument types to be passed to the client event
* @template EventName - A string or union representing the client event name
*
* @param {EventName} eventName - The name of the client event to be called
* @param {Arguments} [args] - Optional arguments to pass to the client event
* @returns {void}
*
* @example
* // Calls an event on a client without specifying a player
* callClient<[], string, object>('onDataRequest')
*/
callClient<Arguments extends unknown[] = unknown[], EventName extends string = string>(eventName: EventName, args?: Arguments): void;
/**
* Calls a client-side event from server or server. Use 'call' to call client from client
*
* @template Arguments - An array of argument types to be passed to the client event
* @template EventName - A string or union representing the client event name
*
* @param {PlayerMp} player - The player for whom the client event is called
* @param {EventName} eventName - The name of the client event to be called
* @param {Arguments} [args] - Optional arguments to pass to the client event
* @returns {void} A promise resolving to the return value of the client event
*
* @example
* // Calls an event on a client for a specific player
* callClient<[string, number], string, boolean>(player, 'onPlayerAction', ['jump', 2])
*/
callClient<Arguments extends unknown[] = unknown[], EventName extends string = string>(player: PlayerMp, eventName: EventName, args?: Arguments): void;
/**
* Calls an asynchronous client-side event from server or browser, expecting a response. Use 'callAsync' to call client from client
*
* @template Arguments - An array of argument types to be passed to the client event
* @template EventName - A string or union representing the client event name
* @template Return - The type of the value returned by the client event
*
* @param {EventName} eventName - The name of the client event to be called
* @param {Arguments} [args] - Optional arguments to pass to the client event
* @returns {Promise<Return>} A promise resolving to the return value of the client event
*
* @example
* // Calls an event on a client without specifying a player
* callClient<[], string, object>('onDataRequest').then(response => {
*   console.log(`Received: ${response}`) //             ^ object
* })
*/
callClientAsync<Arguments extends unknown[] = unknown[], EventName extends string = string, Return = unknown>(eventName: EventName, args?: Arguments): Promise<Return>;
/**
* Calls an asynchronous client-side event from server or browser, expecting a response. Use 'callAsync' to call client from client
*
* @template Arguments - An array of argument types to be passed to the client event
* @template EventName - A string or union representing the client event name
* @template Return - The type of the value returned by the client event
*
* @param {PlayerMp} player - The player for whom the client event is called
* @param {EventName} eventName - The name of the client event to be called
* @param {Arguments} [args] - Optional arguments to pass to the client event
* @returns {Promise<Return>} A promise resolving to the return value of the client event
*
* @example
* // Calls an event on a client for a specific player
* callClient<[string, number], string, boolean>(player, 'onPlayerAction', ['jump', 2]).then(result => {
*   console.log(`Action success: ${result}`) //                                              ^ boolean
* })
*/
callClientAsync<Arguments extends unknown[] = unknown[], EventName extends string = string, Return = unknown>(player: PlayerMp, eventName: EventName, args?: Arguments): Promise<Return>;
/**
* Calls a server-side event from browser or client. Use 'call' to call server from server
*
* @template Arguments - An array of argument types to be passed to the server event
* @template EventName - A string or union representing the server event name
*
* @param {EventName} eventName - The name of the server event to be called
* @param {Arguments} [args] - Optional arguments to pass to the server event
* @returns {void}
*
* @example
* // Calls an event on a server
* callServer<[], string, object>('onDataRequest')
*/
callServer<Arguments extends unknown[] = unknown[], EventName extends string = string>(eventName: EventName, args?: Arguments): void;
/**
* Calls an asynchronous server-side event from browser or client, expecting a response. Use 'callAsync' to call server from server
*
* @template Arguments - An array of argument types to be passed to the server event
* @template EventName - A string or union representing the server event name
* @template Return - The type of the value returned by the server event
*
* @param {EventName} eventName - The name of the server event to be called
* @param {Arguments} [args] - Optional arguments to pass to the server event
* @returns {Promise<Return>} A promise resolving to the return value of the server event
*
* @example
* // Calls an event on a server
* callServer<[], string, object>('onDataRequest').then(response => {
*   console.log(`Received: ${response}`) //             ^ object
* })
*/
callServerAsync<Arguments extends unknown[] = unknown[], EventName extends string = string, Return = unknown>(eventName: EventName, args?: Arguments): Promise<Return>;
/**
* Calls a browser-side event from client or server. Use 'call' to call browser from browser
*
* @template Arguments - An array of argument types to be passed to the browser event
* @template EventName - A string or union representing the browser event name
*
* @param {EventName} eventName - The name of the browser event to be called
* @param {Arguments} [args] - Optional arguments to pass to the browser event
* @returns {void}
*
* @example
* // Calls an event on a browser without specifying a player
* callBrowser<[], string, object>('onDataRequest')
*/
callBrowser<Arguments extends unknown[] = unknown[], EventName extends string = string>(eventName: EventName, args?: Arguments): void;
/**
* Calls a browser-side event from client or server. Use 'call' to call browser from browser
*
* @template Arguments - An array of argument types to be passed to the browser event
* @template EventName - A string representing the browser event name or union of names
*
* @param {PlayerMp} player - The player for whom the browser event is called
* @param {EventName} eventName - The name of the browser event to be called
* @param {Arguments} [args] - Optional arguments to pass to the browser event
* @returns {void}
*
* @example
* // Calls an event on a browser for a specific player
* callBrowser<[string, number], string, boolean>(player, 'onPlayerAction', ['jump', 2])
*/
callBrowser<Arguments extends unknown[] = unknown[], EventName extends string = string>(player: PlayerMp, eventName: EventName, args?: Arguments): void;
/**
* Calls an asynchronous browser-side event from client or server, expecting response. Use 'callAsync' to call browser from browser
*
* @template Arguments - An array of argument types to be passed to the browser event
* @template EventName - A string or union representing the browser event name
* @template Return - The type of the value returned by the browser event
*
* @param {EventName} eventName - The name of the browser event to be called
* @param {Arguments} [args] - Optional arguments to pass to the browser event
* @returns {Promise<Return>} A promise resolving to the return value of the browser event
*
* @example
* // Calls an event on a browser without specifying a player
* callBrowser<[], string, object>('onDataRequest').then(response => {
*   console.log(`Received: ${response}`) //             ^ object
* })
*/
callBrowserAsync<Arguments extends unknown[] = unknown[], EventName extends string = string, Return = unknown>(eventName: EventName, args?: Arguments): Promise<Return>;
/**
* Calls an asynchronous browser-side event from client or server, expecting response. Use 'callAsync' to call browser from browser
*
* @template Arguments - An array of argument types to be passed to the browser event
* @template EventName - A string representing the browser event name or union of names
* @template Return - The type of the value returned by the browser event
*
* @param {PlayerMp} player - The player for whom the browser event is called
* @param {EventName} eventName - The name of the browser event to be called
* @param {Arguments} [args] - Optional arguments to pass to the browser event
* @returns {Promise<Return>} A promise resolving to the return value of the browser event
*
* @example
* // Calls an asynchronous event on a browser for a specific player
* callBrowser<[string, number], string, boolean>(player, 'onPlayerAction', ['jump', 2]).then(result => {
*   console.log(`Action success: ${result}`) //                                              ^ boolean
* })
*/
callBrowserAsync<Arguments extends unknown[] = unknown[], EventName extends string = string, Return = unknown>(player: PlayerMp, eventName: EventName, args?: Arguments): Promise<Return>;
/**
* Calls an event in current environment
*
* @template Arguments - An array of argument types to be passed to the event
* @template EventName - A string or union representing the event name
*
* @param {EventName} eventName - The name of the event to be called
* @param {Arguments} [args] - Optional arguments to pass to the event
* @returns {void}
*
* @example
* // Calls an event in current environment
* call<[], string>('getSomething')
*/
call<Arguments extends unknown[] = unknown[], EventName extends string = string>(eventName: EventName, args?: Arguments): void;
/**
* Calls an asynchronous event in current environment, expecting a response
*
* @template Arguments - An array of argument types to be passed to the event
* @template EventName - A string or union representing the event name
* @template Return - The type of the value returned by the event
*
* @param {EventName} eventName - The name of the event to be called
* @param {Arguments} [args] - Optional arguments to pass to the event
* @returns {Promise<Return>} A promise resolving to the return value of the event
*
* @example
* // Calls an event in current environment
* call<[], string, number>('getSomething').then(response => {
*   console.log(`Received: ${response}`) //      ^ number
* })
*/
callAsync<Arguments extends unknown[] = unknown[], EventName extends string = string, Return = unknown>(eventName: EventName, args?: Arguments): Promise<Return>;
/**
* redirects an event in cases of it calling its own environment
*/
private callSelf;
/**
* redirects an asynchronous event in cases of it calling its own environment
*/
private callSelfAsync;
/**
* returns cross-environment response
*/
private responseHandler;
}

export declare interface RpcConfig extends RpcWrapperConfig {
    debugLogs?: boolean;
}

export declare enum RPCEventType {
    EVENT = "event",
    RESPONSE = "response"
}

export declare type RPCState = {
    eventName: string;
    uuid: string;
    calledFrom: Environment;
    calledTo: Environment;
    knownError?: string;
    data?: any;
    type: RPCEventType;
};

export declare interface RpcWrapperConfig {
    forceBrowserDevMode?: boolean;
}

/**
 * NOT INTENDED FOR OUT-OF-CONTEXT USE
 */
export declare class Server extends Wrapper {
    constructor(options?: RpcWrapperConfig);
    /**
     * NOT INTENDED FOR OUT-OF-CONTEXT USE
     */
    _resolveEmitDestination(player: PlayerMp, dataRaw: string): void;
    private emitClient;
    private emit;
}

export declare class Utils {
    static getEnvironment(): Environment;
    static prepareExecution(data: string): RPCState;
    static prepareTransfer(data: RPCState): string;
    static generateUUID(): string;
    static generateResponseEventName(uuid: string): string;
    static errorUnknownEnvironment(environment: Environment): void;
}

export declare class Wrapper {
    protected environment_: Environment;
    protected state_: any;
    protected console_: {
        (...data: any[]): void;
        (message?: any, ...optionalParams: any[]): void;
    };
    protected debug_: boolean;
    protected forceBrowserDevMode_: boolean;
    constructor(options?: RpcWrapperConfig);
    protected verifyEvent_(data: string | RPCState): RPCState;
    protected triggerError_(rpcData: RPCState, error?: any): void;
    protected log(method: string, eventName: string, ...args: unknown[]): void;
}

export declare const rpc: Rpc
